@@ -1,6 +1,6 @@
 /*
  * battfink-apm.c
- 
+ * 
  * Copyright (C) 2003 Sun Microsystems, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -54,6 +54,14 @@
 #define BATTERY_FLAGS_ABSENT    (0x80)
 #endif
 
+#ifndef BATTERY_PERCENTAGE_UNKNOWN
+#define BATTERY_PERCENTAGE_UNKNOWN  (-1)
+#endif
+
+#ifndef BATTERY_TIME_UKNOWN
+#define BATTERY_TIME_UNKNOWN        (-1)
+#endif
+
 /* These are the headers that are in apm.h - define above 
  * as we need them.
  *
@@ -70,8 +78,6 @@
  * #define BATTERY_FLAGS_LOW       (0x2)
  * #define BATTERY_FLAGS_CRITICAL  (0x4)
  * #define BATTERY_FLAGS_ABSENT    (0x80)
- * #define BATTERY_PERCENTAGE_UNKNOWN  (-1)
- * #define BATTERY_TIME_UNKNOWN        (-1)
  */
 
 typedef enum {
@@ -113,6 +119,16 @@ enum {
   BATTFINK_SIG_LAST
 };
 
+#define BATTFINK_STOCK_HIGH      "battery-high"
+#define BATTFINK_STOCK_NORM      "battery-norm"
+#define BATTFINK_STOCK_LOW       "battery-low"
+#define BATTFINK_STOCK_CRIT      "battery-crit"
+#define BATTFINK_STOCK_AC        "battery-ac"
+
+#define BATTFINK_DEFAULT_ICON_SIZE 16
+static GtkIconSize battfink_icon_size = 0;
+static gboolean battfink_icons_initialized = FALSE;
+
 void battfink_apm_read_info (BattfinkApm *battfink_apm, GError **error);
 
 static GObjectClass *parent_class;
@@ -152,7 +168,7 @@ battfink_apm_read_info (BattfinkApm *battfink_apm, GError **error)
       battfink_apm->priv->battery_status = battery_info.battery_status;
       battfink_apm->priv->battery_percentage = battery_info.battery_percentage;
       battfink_apm->priv->battery_time = battery_info.battery_time;
-      battfink_apm->priv->battery_using_minutes = battery_info.using_minutes ? TRUE : FALSE;
+      battfink_apm->priv->battery_using_minutes = battery_info.using_minutes > 0 ? TRUE : FALSE;
       break;
 
     case 1:
@@ -179,22 +195,6 @@ battfink_apm_read_info (BattfinkApm *battfink_apm, GError **error)
 
 }
 
-static void
-battfink_apm_update_tooltip (BattfinkApm *battfink_apm)
-{
-  gchar *tip;
-  
-  if (battfink_apm->priv->ac_power && battfink_apm->priv->battery_percentage != 100)
-    tip = g_strdup_printf (_("Charging: %d %%"), battfink_apm->priv->battery_percentage);
-  else if (battfink_apm->priv->ac_power && battfink_apm->priv->battery_percentage == 100)
-    tip = g_strdup_printf (_("System is running on AC power"));
-  else
-    tip = g_strdup_printf (_("Battery: %d %%"), battfink_apm->priv->battery_percentage);
-
-  gtk_tooltips_set_tip (GTK_TOOLTIPS (battfink_apm->priv->tooltips), GTK_WIDGET (battfink_apm), tip, NULL);
-  g_free (tip);
-}
-
 static gboolean 
 battfink_apm_animate_charging (BattfinkApm *battfink_apm)
 {
@@ -212,13 +212,9 @@ battfink_apm_get_time (BattfinkApm *battfink_apm)
 { 
   if (battfink_apm->priv->show_time) {
     if (battfink_apm->priv->battery_time / 60 == 0)
-      return g_strdup_printf (_("%d mins"), battfink_apm->priv->battery_time);
-    else if (battfink_apm->priv->battery_time / 60 == 1)
-      return g_strdup_printf (_("%d hour %d mins"),
-                              battfink_apm->priv->battery_time / 60,
-                              battfink_apm->priv->battery_time % 60);
+      return g_strdup_printf ("");
     else
-      return g_strdup_printf (_("%d hours %d mins"),
+      return g_strdup_printf (_("%d:%d"),
                               battfink_apm->priv->battery_time / 60,
                               battfink_apm->priv->battery_time % 60);
   }
@@ -227,31 +223,28 @@ battfink_apm_get_time (BattfinkApm *battfink_apm)
                             battfink_apm->priv->battery_percentage);
 }
 
-static gchar *
-battfink_apm_get_menu_time (BattfinkApm *battfink_apm)
-{ 
-  /* FIXME : This is a hack to reverse the time displayed in the 
-   * menu. There is definitely a nice way to combine this and the
-   * above function - I'm just too lame to figure out what the 
-   * cleverness is
-   */
-  if (!battfink_apm->priv->show_time) {
-    if (battfink_apm->priv->battery_time / 60 == 0)
-      return g_strdup_printf (_("%d mins"), battfink_apm->priv->battery_time);
-    else if (battfink_apm->priv->battery_time / 60 == 1)
-      return g_strdup_printf (_("%d hour %d mins"),
-                              battfink_apm->priv->battery_time / 60,
-                              battfink_apm->priv->battery_time % 60);
-    else
-      return g_strdup_printf (_("%d hours %d mins"),
-                              battfink_apm->priv->battery_time / 60,
-                              battfink_apm->priv->battery_time % 60);
-  }
+static void
+battfink_apm_update_tooltip (BattfinkApm *battfink_apm)
+{
+  gchar *tip;
+  gchar *time;
+
+  time = battfink_apm_get_time (battfink_apm);
+
+  if (battfink_apm->priv->ac_power && battfink_apm->priv->battery_percentage != 100) 
+    tip = g_strdup_printf (_("Charging: %s"), time);
+  else if (battfink_apm->priv->ac_power && battfink_apm->priv->battery_percentage == 100)
+    tip = g_strdup_printf (_("System is running on AC power"));
   else
-    return g_strdup_printf (_("%d %%"),
-                            battfink_apm->priv->battery_percentage);
+    tip = g_strdup_printf (_("Battery: %s"), time);
+
+  gtk_tooltips_set_tip (GTK_TOOLTIPS (battfink_apm->priv->tooltips), GTK_WIDGET (battfink_apm), tip, NULL);
+
+  g_free (time);
+  g_free (tip);
 }
 
+
 static void
 battfink_apm_update_label (BattfinkApm *battfink_apm)
 {
@@ -267,14 +260,21 @@ battfink_apm_update_image (BattfinkApm *battfink_apm)
   static gboolean first_time = FALSE;
   GError *error = NULL;
 
-  battfink_apm_read_info (battfink_apm, &error);
+  if (battfink_apm->priv->ac_power) {
+    if (battfink_apm->priv->show_time)
+      battfink_apm->priv->show_time = FALSE;
+  }
 
-  battfink_apm_update_label (battfink_apm);
+  battfink_apm_read_info (battfink_apm, &error);
 
   if (error) {
     g_warning ("Failed to read APM information : %s", error->message);
     g_error_free (error);
   }
+
+  if (battfink_apm->priv->battery_time != BATTERY_TIME_UNKNOWN)
+    battfink_apm_update_label (battfink_apm);
+
   battfink_apm_update_tooltip (battfink_apm);
 
   /* If the power is on and battery is fully charged, then remove the timeout and show the 
@@ -296,7 +296,7 @@ battfink_apm_update_image (BattfinkApm *battfink_apm)
    */
   if (battfink_apm->priv->charging_timeout == -1 && 
       battfink_apm->priv->ac_power && 
-      battfink_apm->priv->battery_percentage != 100) {
+      battfink_apm->priv->battery_percentage < 100) {
 
     if (!first_time) {
       /* This is a pretty ugly hack to get things to work properly */
@@ -375,33 +375,109 @@ battfink_apm_disable (BattfinkApm *battfink_apm, EggTrayIcon *tray_icon)
   battfink_apm->priv->timeout = -1; 
 }
 
-static void
-battfink_apm_init_pixbuf (GdkPixbuf **pixbuf, const gchar *filename)
+static gint
+battfink_apm_time_left (BattfinkApm *battfink_apm)
 {
   GError *error = NULL;
 
-  /* FIXME: Load as stock please */
-  *pixbuf = gdk_pixbuf_new_from_file (filename, &error);
+  battfink_apm_read_info (battfink_apm, &error);
 
   if (error) {
-    g_warning ("Could not load pixbuf : %s", error->message);
+    g_warning ("Failed to read APM information : %s", error->message);
     g_error_free (error);
+    return -1;
   }
+  
+  if (battfink_apm->priv->ac_power)
+    return -1;
+  else
+    return battfink_apm->priv->battery_time;
+}
+
+typedef struct
+{
+  char *stock_id;
+  const guint8 *icon_data;
+} BattfinkStockIcon;
+
+static BattfinkStockIcon items[] = {
+  { BATTFINK_STOCK_HIGH, BATTFINK_ICONDIR"/battfink-100.png" },
+  { BATTFINK_STOCK_NORM, BATTFINK_ICONDIR"/battfink-75.png" },
+  { BATTFINK_STOCK_LOW, BATTFINK_ICONDIR"/battfink-50.png" },
+  { BATTFINK_STOCK_CRIT,BATTFINK_ICONDIR"/battfink-25.png" },
+  { BATTFINK_STOCK_AC,  BATTFINK_ICONDIR"/battfink-charge.png" }
+};
+
+static void
+battfink_apm_register_stock_icons (GtkIconFactory *factory)
+{
+  GtkIconSource *source;
+  int i;
+  
+  source = gtk_icon_source_new ();
+  for (i = 0; i < G_N_ELEMENTS (items); i++) {
+    GtkIconSet *icon_set;
+  
+    gtk_icon_source_set_filename (source, items[i].icon_data);
+    icon_set = gtk_icon_set_new ();
+    gtk_icon_set_add_source (icon_set, source);
+    gtk_icon_factory_add (factory, items[i].stock_id, icon_set);
+    gtk_icon_set_unref (icon_set);
+  }
+  gtk_icon_source_free (source);
+}
+
+static void
+battfink_apm_init_stock_icons ()
+{
+  GtkIconFactory *factory;
+
+  if (battfink_icons_initialized)
+    return;
+
+  battfink_icon_size = gtk_icon_size_register ("panel-menu", 
+                                               BATTFINK_DEFAULT_ICON_SIZE, 
+                                               BATTFINK_DEFAULT_ICON_SIZE);
+  factory = gtk_icon_factory_new ();
+  gtk_icon_factory_add_default (factory);
+  battfink_apm_register_stock_icons (factory);
+  g_object_unref (factory);
+  battfink_icons_initialized = TRUE;
 }
 
 static void
 battfink_apm_init_icons (BattfinkApm *battfink_apm)
 {
-  battfink_apm_init_pixbuf (&battfink_apm->priv->battery_icon[BATTFINK_APM_HIGH],
-                            BATTFINK_ICONDIR "/battfink-100.png");
-  battfink_apm_init_pixbuf (&battfink_apm->priv->battery_icon[BATTFINK_APM_MEDIUM],
-                            BATTFINK_ICONDIR "/battfink-75.png");
-  battfink_apm_init_pixbuf (&battfink_apm->priv->battery_icon[BATTFINK_APM_LOW],
-                            BATTFINK_ICONDIR "/battfink-50.png");
-  battfink_apm_init_pixbuf (&battfink_apm->priv->battery_icon[BATTFINK_APM_CRITICAL],
-                            BATTFINK_ICONDIR "/battfink-25.png");
-  battfink_apm_init_pixbuf (&battfink_apm->priv->battery_icon[BATTFINK_APM_POWER],
-                            BATTFINK_ICONDIR "/battfink-charge.png");
+  if (battfink_apm->priv->battery_icon[BATTFINK_APM_HIGH])
+    g_object_unref (battfink_apm->priv->battery_icon[BATTFINK_APM_HIGH]);
+  if (battfink_apm->priv->battery_icon[BATTFINK_APM_MEDIUM])
+    g_object_unref (battfink_apm->priv->battery_icon[BATTFINK_APM_MEDIUM]);
+  if (battfink_apm->priv->battery_icon[BATTFINK_APM_LOW])
+    g_object_unref (battfink_apm->priv->battery_icon[BATTFINK_APM_LOW]);
+  if (battfink_apm->priv->battery_icon[BATTFINK_APM_CRITICAL])
+    g_object_unref (battfink_apm->priv->battery_icon[BATTFINK_APM_CRITICAL]);
+  if (battfink_apm->priv->battery_icon[BATTFINK_APM_POWER])
+    g_object_unref (battfink_apm->priv->battery_icon[BATTFINK_APM_POWER]);
+
+  battfink_apm->priv->battery_icon[BATTFINK_APM_HIGH] = gtk_widget_render_icon (GTK_WIDGET (battfink_apm),
+                                                                                BATTFINK_STOCK_HIGH,
+                                                                                battfink_icon_size, NULL);
+
+  battfink_apm->priv->battery_icon[BATTFINK_APM_MEDIUM] = gtk_widget_render_icon (GTK_WIDGET (battfink_apm),
+                                                                                  BATTFINK_STOCK_NORM,
+                                                                                  battfink_icon_size, NULL);
+
+  battfink_apm->priv->battery_icon[BATTFINK_APM_LOW] = gtk_widget_render_icon (GTK_WIDGET (battfink_apm),
+                                                                               BATTFINK_STOCK_LOW,
+                                                                               battfink_icon_size, NULL);
+
+  battfink_apm->priv->battery_icon[BATTFINK_APM_CRITICAL] = gtk_widget_render_icon (GTK_WIDGET (battfink_apm),
+                                                                                    BATTFINK_STOCK_CRIT,
+                                                                                    battfink_icon_size, NULL);
+
+  battfink_apm->priv->battery_icon[BATTFINK_APM_POWER] = gtk_widget_render_icon (GTK_WIDGET (battfink_apm),
+                                                                                 BATTFINK_STOCK_AC,
+                                                                                 battfink_icon_size, NULL);
 }
 
 static void
@@ -452,6 +528,12 @@ battfink_apm_remove_icon_callback (GtkWidget *widget, gpointer user_data)
   g_signal_emit (G_OBJECT (user_data), battfink_apm_signals [BATTFINK_SIG_DISABLE], 0);
 }
 
+static void
+battfink_apm_open_preferences (GtkWidget *widget, gpointer user_data)
+{
+  g_signal_emit (G_OBJECT (user_data), battfink_apm_signals [BATTFINK_SIG_PREFERENCE], 0);
+}
+
 static void
 battfink_apm_popup_menu (BattfinkApm *battfink_apm, guint button, guint32 activate_time)
 {
@@ -470,20 +552,13 @@ battfink_apm_popup_menu (BattfinkApm *battfink_apm, guint button, guint32 activa
 
   menu = gtk_menu_new ();
 
-  if (battfink_apm->priv->battery_time > 0 && battfink_apm->priv->battery_using_minutes
-      && !(battfink_apm->priv->battery_flags & BATTERY_FLAGS_ABSENT)) {
-    gchar *label;
+  menu_item = gtk_menu_item_new_with_mnemonic (_("_Open Energy Saving Preferences"));
+  g_signal_connect (G_OBJECT (menu_item), "activate",
+                    G_CALLBACK (battfink_apm_open_preferences), battfink_apm);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 
-    label = battfink_apm_get_menu_time (battfink_apm);
-
-    menu_item = gtk_menu_item_new_with_label (label);
-    gtk_widget_set_sensitive (GTK_WIDGET (menu_item), FALSE);
-    g_free (label);
-    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-
-    menu_item = gtk_separator_menu_item_new ();
-    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
-  }
+  menu_item = gtk_separator_menu_item_new ();
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 
   if (battfink_apm->priv->battery_time > 0
       && !(battfink_apm->priv->battery_flags & BATTERY_FLAGS_ABSENT)) {
@@ -509,6 +584,8 @@ battfink_apm_popup_menu (BattfinkApm *battfink_apm, guint button, guint32 activa
 
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), battfink_apm->priv->percent_menu_item);
 
+    menu_item = gtk_separator_menu_item_new ();
+    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
   }
 
   menu_item = gtk_image_menu_item_new_with_mnemonic (_("_Remove Icon"));
@@ -596,6 +673,7 @@ battfink_apm_class_init (BattfinkApmClass *klass)
 
   klass->enable  = battfink_apm_enable;
   klass->disable = battfink_apm_disable;
+  klass->time_left = battfink_apm_time_left;
 
   widget_class->button_press_event = battfink_apm_button_press_event;
 
@@ -616,6 +694,16 @@ battfink_apm_class_init (BattfinkApmClass *klass)
                                                                G_TYPE_NONE, 0);
 }
 
+static void
+battfink_apm_style_set (GtkWidget *widget, GtkStyle *prev_style, gpointer user_data)
+{
+  BattfinkApm *battfink_apm;
+
+  battfink_apm = BATTFINK_APM (widget);
+
+  battfink_apm_init_icons (battfink_apm);
+}
+
 static void
 battfink_apm_init (BattfinkApm *battfink_apm,
                    BattfinkApmClass *klass)
@@ -641,6 +729,10 @@ battfink_apm_init (BattfinkApm *battfink_apm,
   gtk_widget_add_events (GTK_WIDGET (battfink_apm), 
                          GDK_BUTTON_PRESS_MASK | GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
 
+  g_signal_connect (battfink_apm, "style-set",
+                    G_CALLBACK (battfink_apm_style_set), NULL);
+
+  battfink_apm_init_stock_icons ();
   battfink_apm_init_icons (battfink_apm);
 
   battfink_apm_read_info (battfink_apm, &error);
